import { AnnotationComponent, createAnnotation, deleteNode } from 'substance'

class TrackChangeComponent extends AnnotationComponent {
  render ($$) {
    const { status } = this.props.node
    const trackChangesView = this.toggleTrackChangeView()

    const accept = $$('a')
      .addClass('sc-track-item')
      .on('click', this.acceptTrackChange)
      .append('accept')

    const reject = $$('a')
      .addClass('sc-track-item')
      .on('click', this.rejectTrackChange)
      .append('reject')

    const separator = $$('span')
      .addClass('sc-track-separator')
      .append(' / ')

    const container = $$('span')
      .addClass('sc-accept-reject-container')
      .append(accept)
      .append(separator)
      .append(reject)

    let el = $$('span')
      .addClass(this.getClassNames())
      .append(this.props.children)
      .append(container)

    el.addClass('sc-track-change-' + status)

    if (trackChangesView === false) {
      if (status === 'delete') el.addClass('sc-track-delete-hide')
      if (status === 'add') el.addClass('sc-track-add-show')
    }

    return el
  }

// TODO Move most of the funcs into provider
  getSurface () {
    const surfaceManager = this.context.surfaceManager
    const containerId = this.context.controller.props.containerId

    return surfaceManager.getSurface(containerId)
  }

  toggleTrackChangeView () {
    const self = this
    const surface = self.getSurface()

    if (!surface) return
    return surface._owner.state.trackChangesView
  }

  acceptTrackChange () {
    const nodeId = this.props.node.id
    const status = this.props.node.status
    const ds = this.getDocumentSession()
    const doc = ds.getDocument()
    const nodeData = this.buildNodeData()

    var self = this

    ds.transaction(function (tx, args) {
      if (status === 'add') createAnnotation(doc, nodeData)
      deleteNode(tx, { nodeId: nodeId })
      if (status === 'delete') self.context.surface.delete(tx, nodeData)
    })
  }

  rejectTrackChange () {
    const nodeId = this.props.node.id
    const status = this.props.node.status
    const ds = this.getDocumentSession()
    const doc = ds.getDocument()
    const nodeData = this.buildNodeData()

    var self = this

    ds.transaction(function (tx, args) {
      if (status === 'delete') createAnnotation(doc, nodeData)
      deleteNode(tx, { nodeId: nodeId })
      if (status === 'add') self.context.surface.delete(tx, nodeData)
    })
  }

  buildNodeData () {
    const nodeId = this.props.node.id
    const ds = this.getDocumentSession()
    const doc = ds.getDocument()
    const trackChangeNode = doc.get(nodeId)

    const path = trackChangeNode.path
    const startOffset = trackChangeNode.startOffset
    const endOffset = trackChangeNode.endOffset

    const sel = ds.createSelection(path, startOffset, endOffset)

    return {
      selection: sel,
      node: { type: 'paragraph' }
    }
  }

  getDocumentSession () {
    return this.context.documentSession
  }

}

export default TrackChangeComponent
